@@ -1,5 +1,6 @@
 
-from hgsvn.errors import ExternalCommandFailed
+from hgsvn import ui
+from hgsvn.errors import ExternalCommandFailed, HgSVNError
 
 import os
 import locale
@@ -16,21 +17,39 @@ try:
 except ImportError:
     commands = None
 
+class _SimpleFileLock(object):
+
+    def __init__(self, filename, timeout=None):
+        self.file = filename
+        self.lock()
+
+    def lock(self):
+        if os.path.exists(self.file):
+            raise LockHeld("Lock file exists.")
+        open(self.file, "wb").close()
+
+    def release(self):
+        if os.path.exists(self.file):
+            os.remove(self.file)
+
+
+class _LockHeld(Exception):
+    pass
+
+
 # We import the lock logic from Mercurial if it is available, and fall back
 # to a dummy (always successful) lock if not.
 try:
-    from mercurial.lock import lock as _lock, LockHeld
+    from mercurial.lock import lock as _lock
+    try:
+        from mercurial.error import LockHeld
+    except ImportError:
+        # LockHeld was defined in mercurial.lock in Mercurial < 1.2
+        from mercurial.lock import LockHeld
+
 except ImportError:
-    class _StubLock(object):
-        def __init__(self, file, timeout=None):
-            pass
-        def lock(self):
-            pass
-        def release(self):
-            pass
-    _lock = _StubLock
-    class LockHeld(Exception):
-        pass
+    _lock = _SimpleFileLock
+    LockHeld = _LockHeld
 
 
 hgsvn_private_dir = ".hgsvn"
@@ -108,7 +127,7 @@ def shell_quote(s):
 
 def _run_raw_command(cmd, args, fail_if_stderr=False):
     cmd_string = "%s %s" % (cmd,  " ".join(map(shell_quote, args)))
-    print "*", cmd_string
+    ui.status("* %s", cmd_string, level=ui.DEBUG)
     try:
         pipe = Popen([cmd] + args, executable=cmd, stdout=PIPE, stderr=PIPE)
     except OSError:
@@ -124,7 +143,7 @@ def _run_raw_command(cmd, args, fail_if_stderr=False):
     return out
 
 def _run_raw_shell_command(cmd):
-    print "*", cmd
+    ui.status("* %s", cmd, level=ui.DEBUG)
     st, out = commands.getstatusoutput(cmd)
     if st != 0:
         raise ExternalCommandFailed(
@@ -150,7 +169,10 @@ def run_command(cmd, args=None, bulk_args=None, encoding=None, fail_if_stderr=Fa
     # If one of bulk_args starts with a slash (e.g. '-foo.php'),
     # hg and svn will take this as an option. Adding '--' ends the search for
     # further options.
-    args.append("--")
+    for a in bulk_args:
+        if a.strip().startswith('-'):
+            args.append("--")
+            break
     max_args_num = 254
     i = 0
     out = ""
@@ -295,3 +317,53 @@ def get_hgsvn_lock(basedir='.'):
     return _lock(os.path.join(basedir, hgsvn_private_dir, hgsvn_lock_file),
         timeout=0)
 
+
+def change_to_rootdir():
+    """Changes working directory to root of checkout.
+
+    Raises HgSVNError if the current working directory isn't a Mercurial
+    repository.
+    """
+    try:
+        root = run_hg(["root"]).strip()
+    except ExternalCommandFailed, err:
+        raise HgSVNError('No Mercurial repository found.')
+    os.chdir(root)
+
+
+def hg_is_clean(current_branch):
+    """Returns False if the local Mercurial repository has
+       uncommitted changes."""
+    if run_hg(['st', '-mard']).strip():
+        ui.status(("\nThe Mercurial working copy contains pending changes "
+                   "in branch '%s'!\n"
+                   "Please either commit or discard them before running "
+                   "'%s' again."
+                   % (current_branch, get_script_name())),
+                  level=ui.ERROR)
+        return False
+    return True
+
+
+def get_script_name():
+    """Helper to return the name of the command line script that was called."""
+    return os.path.basename(sys.argv[0])
+
+
+def hg_switch_branch(current_branch, new_branch):
+    """Safely switch the Mercurial branch.
+
+    The function will only switch to new_branch iff there are no uncommitted
+    changed in the  current branch.
+
+    True is returned if the switch was successful otherwise False.
+    """
+    hg_branches = [l.split()[0] for l in run_hg(["branches"]).splitlines()]
+    if new_branch in hg_branches:
+        # We want to run "hg up -C" (to force changing branches) but we
+        # don't want to erase uncommitted changes.
+        if not hg_is_clean(current_branch):
+            return False
+        run_hg(['up', '-C', new_branch])
+    run_hg(["branch", new_branch])
+    return True
