@@ -67,6 +67,10 @@ def main():
     if os.path.exists(local_repo):
         if not os.path.isdir(local_repo):
             raise ValueError("%s is not a directory" % local_repo)
+        if os.path.isdir(os.path.join(local_repo, '.svn')):
+            nlocal_repo = os.path.abspath(local_repo)
+            print "%s is already a SVN checkout." % nlocal_repo
+            sys.exit(1)
     else:
         os.mkdir(local_repo)
     fixup_hgsvn_dir(local_repo)
