@@ -4,25 +4,24 @@ Mercurial changesets.
 
 """
 
+from hgsvn import ui
 from hgsvn.common import (
-    run_command, run_hg, run_svn, skip_dirs,
-    hg_commit_from_svn_log_entry, hg_tag_svn_rev, hg_exclude_options,
-    hg_commit_prefix, get_svn_rev_from_hg, get_hgsvn_lock, LockHeld,
-    fixup_hgsvn_dir,
+    run_hg, run_svn,
+    hg_commit_from_svn_log_entry, hg_exclude_options,
+    get_svn_rev_from_hg, hg_switch_branch, hg_is_clean,
 )
 from hgsvn.svnclient import (
-    get_svn_info, get_svn_versioned_files,
-    iter_svn_log_entries,
+    get_svn_info, get_svn_versioned_files, iter_svn_log_entries, run_svn_log,
+    get_svn_client_version, get_svn_status
 )
 from hgsvn.errors import (
-    HgSVNError, ExternalCommandFailed, OverwrittenSVNBranch, UnsupportedSVNAction,
+    ExternalCommandFailed, OverwrittenSVNBranch, UnsupportedSVNAction,
 )
 from hgsvn.run.common import run_parser, display_parser_error
+from hgsvn.run.common import locked_main
 
 import sys
 import os
-import shutil
-import tempfile
 import time
 import traceback
 from optparse import OptionParser
@@ -125,7 +124,19 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
     # Update SVN + add/remove/commit hg
     try:
         if changed_paths:
-            run_svn(["up", "--ignore-externals", "-r", svn_rev, svn_wc])
+            args = ["up", "--ignore-externals"]
+            if get_svn_client_version() >= (1, 5):
+                args.extend(['--accept', 'postpone'])
+            run_svn(args + ["-r", svn_rev, svn_wc])
+            conflicts = []
+            for status_entry in get_svn_status('.'):
+                if status_entry['status'] == 'conflicted':
+                    conflicts.append(status_entry['path'])
+            if conflicts:
+                ui.status('SVN updated resulted in conflicts!', level=ui.ERROR)
+                ui.status('Conflicted files: %s', ','.join(conflicts))
+                ui.status('Please report a bug.')
+                return None
             for p, old_contents in replaced_paths.items():
                 old_contents = set(old_contents)
                 new_contents = set(get_svn_versioned_files(
@@ -155,7 +166,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
             detect_overwritten_svn_branch(wc_url, svn_rev)
     # NOTE: in Python 2.5, KeyboardInterrupt isn't a subclass of Exception anymore
     except (Exception, KeyboardInterrupt), e:
-        print "\nInterrupted, please wait for cleanup!\n"
+        ui.status("\nInterrupted, please wait for cleanup!\n", level=ui.ERROR)
         # NOTE: at this point, running SVN sometimes gives "write lock failures"
         # which leave the WC in a weird state.
         time.sleep(0.3)
@@ -170,15 +181,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
     return svn_rev
 
 
-def locked_main():
-    usage = "usage: %prog [-p SVN peg rev] [--help]"
-    parser = OptionParser(usage)
-    parser.add_option("-p", "--svn-peg", type="int", dest="svn_peg",
-       help="SVN peg revision to locate checkout URL")
-    (options, args) = run_parser(parser, __doc__)
-    if args:
-        display_parser_error(parser, "incorrect number of arguments")
-
+def real_main(options, args):
     svn_wc = "."
     svn_peg_wc = svn_wc
     if options.svn_peg:
@@ -200,27 +203,35 @@ def locked_main():
     if options.svn_peg:
        wc_url += "@" + str(options.svn_peg)
 
-    # Get remove SVN info
+    # Get remote SVN info
+    ui.status("Retrieving remote SVN info...", level=ui.VERBOSE)
     svn_greatest_rev = get_svn_info(wc_url)['last_changed_rev']
     if svn_greatest_rev < next_rev:
-        print "No revisions after %s in SVN repo, nothing to do" % svn_greatest_rev
+        ui.status("No revisions after %s in SVN repo, nothing to do",
+                  svn_greatest_rev)
+        return
+
+    # Show incoming changes if in dry-run mode.
+    if options.dryrun:
+        ui.status("Incoming SVN revisions:")
+        for entry in run_svn_log(repos_url, current_rev, svn_greatest_rev,
+                                 100):
+            msg = entry["message"].splitlines()[0]
+            line = "[%d] %s" % (entry["revision"], msg)
+            # TODO: Line stripping should be done by some simple UI interface.
+            if len(line) > 79:
+                line = line[:77]+"..."
+            ui.status(line)
         return
 
     # Prepare and/or switch named branches
     orig_branch = run_hg(["branch"]).strip()
     if orig_branch != svn_branch:
         # Update to or create the "pristine" branch
-        hg_branches = [l.split()[0] for l in run_hg(["branches"]).splitlines()]
-        if svn_branch in hg_branches:
-            # We want to run "hg up -C" (to force changing branches) but we
-            # don't want to erase uncommitted changes.
-            if run_hg(['st', '-mard']).strip():
-                print ("\nThe Mercurial working copy contains pending changes in branch '%s'!\n"
-                    + "Please either commit or discard them before running 'hgpullsvn' again."
-                    ) % orig_branch
-                return 1
-            run_hg(['up', '-C', svn_branch])
-        run_hg(["branch", svn_branch])
+        if not hg_switch_branch(orig_branch, svn_branch):
+            return 1
+    elif not hg_is_clean(svn_branch):
+        return 1
 
     # Detect that a previously aborted hgpullsvn retrieved an SVN revision
     # without committing it to hg.
@@ -228,49 +239,70 @@ def locked_main():
     # during the previous "hg tag".
     hgsvn_rev = get_svn_rev_from_hg()
     if hgsvn_rev is not None and hgsvn_rev != current_rev:
-        print ("\nNote: hgsvn repository is in an unclean state "
-            + "(probably because of an aborted hgpullsvn). \n"
-            + "Let's first update to the last good SVN rev.")
+        ui.status(("\nNote: hgsvn repository is in an unclean state "
+                   "(probably because of an aborted hgpullsvn). \n"
+                   "Let's first update to the last good SVN rev."),
+                  level=ui.VERBOSE)
         run_svn(["revert", "-R", "."])
         run_svn(["up", "--ignore-externals", "-r", hgsvn_rev, svn_wc])
         next_rev = hgsvn_rev + 1
 
+    # Reset working branch to last svn head to have a clean and linear SVN
+    # histoy.
+    heads_before = None
+    if hgsvn_rev is None:
+        heads_before = run_hg(["heads", "--template",
+                               "{node}%s" % os.linesep]).splitlines()
+        run_hg(["update", "-C", "svn.%d" % current_rev])
+
     # Load SVN log starting from current rev
     it_log_entries = iter_svn_log_entries(svn_peg_wc, next_rev, svn_greatest_rev)
 
     try:
         try:
             for log_entry in it_log_entries:
-                current_rev = pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base)
+                current_rev = pull_svn_rev(log_entry, current_rev,
+                                           svn_wc, wc_url, wc_base)
+                if current_rev is None:
+                    return 1
+                ui.status("Pulled r%d %s (%s)", log_entry["revision"],
+                          log_entry["message"], log_entry["author"])
+
         # TODO: detect externals with "svn status" and update them as well
 
         finally:
+            if heads_before is not None:  # we have reset the SVN branch
+                heads_now = run_hg(["heads", "--template",
+                                    "{node}%s" % os.linesep]).splitlines()
+                if len(heads_now) != len(heads_before):
+                    ui.status("created new head in branch '%s'", svn_branch)
             work_branch = orig_branch or svn_branch
             if work_branch != svn_branch:
                 run_hg(["up", '-C', work_branch])
                 run_hg(["branch", work_branch])
 
     except KeyboardInterrupt:
-        print "\nStopped by user."
+        ui.status("\nStopped by user.", level=ui.ERROR)
     except ExternalCommandFailed, e:
-        print str(e)
+        ui.status(str(e), level=ui.ERROR)
     except:
-        print "\nCommand failed with following error:\n"
+        ui.status("\nCommand failed with following error:\n", level=ERROR)
         traceback.print_exc()
 
 
 def main():
-    # We must do this before trying to take the lock file
-    fixup_hgsvn_dir('.')
-    try:
-        l = get_hgsvn_lock()
-    except LockHeld, e:
-        print "Aborting:", e
-        return 1
-    try:
-        return locked_main()
-    finally:
-        l.release()
+    # Defined as entry point. Must be callable without arguments.
+    usage = "usage: %prog [-p SVN peg rev] [--help]"
+    parser = OptionParser(usage)
+    parser.add_option("-p", "--svn-peg", type="int", dest="svn_peg",
+       help="SVN peg revision to locate checkout URL")
+    parser.add_option("-n", "--dry-run", dest="dryrun", default=False,
+                      action="store_true",
+                      help="show incoming changesets without pulling them")
+    (options, args) = run_parser(parser, __doc__)
+    if args:
+        display_parser_error(parser, "incorrect number of arguments")
+    return locked_main(real_main, options, args)
 
 
 if __name__ == "__main__":
