@@ -1,4 +1,5 @@
 
+from hgsvn import ui
 from hgsvn.common import run_svn
 from hgsvn.errors import EmptySVNLog
 
@@ -76,12 +77,21 @@ def parse_svn_log_xml(xml_string):
     for entry in tree.findall('logentry'):
         d = {}
         d['revision'] = int(entry.get('revision'))
-        author = entry.find('author')
         # Some revisions don't have authors, most notably the first revision
         # in a repository.
+        # logentry nodes targeting directories protected by path-based
+        # authentication have no child nodes at all. We return an entry
+        # in that case. Anyway, as it has no path entries, no further
+        # processing will be made.
+        author = entry.find('author')
+        date = entry.find('date')
+        msg = entry.find('msg')
         d['author'] = author is not None and author.text or None
-        d['date'] = svn_date_to_timestamp(entry.find('date').text)
-        d['message'] = entry.find('msg').text or ""
+        if date is not None:
+            d['date'] = svn_date_to_timestamp(date.text)
+        else:
+            d['date'] = None
+        d['message'] = msg is not None and msg.text or ""
         paths = d['changed_paths'] = []
         for path in entry.findall('.//path'):
             copyfrom_rev = path.get('copyfrom-rev')
@@ -120,6 +130,7 @@ def parse_svn_status_xml(xml_string, base_dir=None):
             d['type'] = 'normal'
         else:
             d['type'] = 'unversioned'
+        d['status'] = wc_status.get('item')
         l.append(d)
     return l
 
@@ -228,6 +239,8 @@ def iter_svn_log_entries(svn_url, first_rev, last_rev):
     while last_rev == "HEAD" or cur_rev <= last_rev:
         start_t = time.time()
         stop_rev = min(last_rev, cur_rev + int(chunk_length * chunk_interval_factor))
+        ui.status("Fetching SVN log for revisions %d-%d...", cur_rev, stop_rev,
+                  level=ui.VERBOSE)
         entries = run_svn_log(svn_url, cur_rev, stop_rev, chunk_length)
         duration = time.time() - start_t
         if not entries:
@@ -246,3 +259,17 @@ def iter_svn_log_entries(svn_url, first_rev, last_rev):
             chunk_length = max(log_min_chunk_length, int(chunk_length / 2.0))
 
 
+_svn_client_version = None
+
+def get_svn_client_version():
+    """Returns the SVN client version as a tuple.
+
+    The returned tuple only contains numbers, non-digits in version string are
+    silently ignored.
+    """
+    global _svn_client_version
+    if _svn_client_version is None:
+        raw = run_svn(['--version', '-q']).strip()
+        _svn_client_version = tuple(map(int, [x for x in raw.split('.')
+                                              if x.isdigit()]))
+    return _svn_client_version
